package softmc;

import javassist.ClassPool;
import korat.instrumentation.cfg.CFGBuilder;
import korat.instrumentation.cfg.ast.CFG;
import korat.instrumentation.cfg.ast.CFGCollection;
import softmc.logic.Term;
import softmc.state.GlobalState;

public class ModelChecker {

    public ModelChecker() {
    }

    public void checkMethod(String className, String methodName) throws Exception {
        GlobalState state = convertToState(className, methodName);
        // Uncomment to generate a dot diagram for the state graph
        //Utils.dot2Pdf(state.toDot(), "symStatesOpt.pdf");

        States2ListLogic converter = new States2ListLogic(state);
        Term initConstr = converter.convertToInitConstr();
        Term trFn = converter.convertToTransitionFunction();

        System.out.println("Initial constraint: ");
        System.out.println(initConstr.toString());        
        System.out.println();
        
        System.out.println("Transition constraint:");
        System.out.println(trFn.toString());
        System.out.println();
        
        String name = new String(new char[] { methodName.charAt(0) }).toUpperCase()
                + methodName.substring(1);

        Z3Converter z3Conv = new Z3Converter(name, state, initConstr, trFn,
                GlobalOptions.__templ_file);
        z3Conv.convert();        
    }

    public GlobalState convertToState(String className, String methodName) throws Exception {
        CFGBuilder b = new CFGBuilder();
        CFGCollection cfgCol = b.build(className, methodName);
        CFG cfg = cfgCol.getCFGMap().values().iterator().next().values().iterator().next();
        ClassPool classPool = b.getClassPool();

        CFGPreProcessor preProcessor = new CFGPreProcessor(classPool);
        preProcessor.process(cfg);

        CFG2StatesConverter conv = new CFG2StatesConverter(classPool);
        GlobalState state = conv.convertToStates(cfg);

        StateOptimizer opt = new StateOptimizer();
        opt.removeEmptyStates(state);
//        opt.minimize(state);
//        opt.unfold(state);
        // opt.simplifyTerms(state);

        return state;
    }

}
