package softmc.examples;

import softmc.Axioms;
import korat.instrumentation.cfg.annotations.CFGAnalyzeMe;

@CFGAnalyzeMe
public class Examples {

    public static void simpleWhile0(final int N) {
        int x = 0;
        int i = 0;
        while (i < N) {
            x += 2;
            i++;
        }
        assert x == 2*N;
    }
    
    public static void simpleWhile(final int N) {
        int x = 0;
        int i = 0;
        while (i < N) {
            if (i % 2 == 0)
                x += 2;
            i++;
        }
        assert x == N || x == N + 1;
    }

    public static void bblSort(final int N, int[] a) {
        for (int j = 0; j < N - 1; j++) {
            for (int i = 0; i < N - j - 1; i++) {
                if (a[i] > a[i + 1]) {
                    int t = a[i];
                    a[i] = a[i + 1];
                    a[i + 1] = t;
                }
            }
        }
        for (int j = 0; j < N - 1; j++)
            assert a[j] <= a[j + 1];
    }
    
    public static void bblSort_bug1(final int N, int[] a) {
        for (int j = 0; j < N - 1; j++) {
            for (int i = 0; i < N - j - 1; i++) {
                if (a[i] > a[i + 1]) {
                    int t = a[i];
                    a[i] = a[i + 1];
                    a[i + 1] = t;
                }
            }
        }
        for (int j = 0; j < N - 1; j++)
            assert a[j] < a[j + 1]; // BUG
    }
    
    public static void bblSort_bug2(final int N, int[] a) {
        for (int j = 0; j < N - 2; j++) { // BUG
            for (int i = 0; i < N - j - 1; i++) {
                if (a[i] > a[i + 1]) {
                    int t = a[i];
                    a[i] = a[i + 1];
                    a[i + 1] = t;
                }
            }
        }
        for (int j = 0; j < N - 1; j++)
            assert a[j] <= a[j + 1];
    }
    
    public static void bblSort_bug3(final int N, int[] a) {
        for (int j = 0; j < N - 1; j++) {
            for (int i = 1; i < N - j - 1; i++) { // BUG
                if (a[i] > a[i + 1]) {
                    int t = a[i];
                    a[i] = a[i + 1];
                    a[i + 1] = t;
                }
            }
        }
        for (int j = 0; j < N - 1; j++)
            assert a[j] <= a[j + 1];
    }
    
    public static void bblSort_ax(final int N, int[] a) {
        for (int j = 0; j < N - 1; j++) {
            for (int i = 0; i < N - j - 1; i++) {
                if (a[i] > a[i + 1]) {
                    int t = a[i];
                    a[i] = a[i + 1];
                    a[i + 1] = t;
                }
            }
        }
        assert Axioms.isSorted(a, 0, N);
    }

    public static void selectSort(final int N, int[] a) {
        for (int j = 0; j < N - 1; j++) {
            int min = j;
            for (int i = j + 1; i < N; i++) {
                if (a[min] > a[i])
                    min = i;
            }
            int t = a[j];
            a[j] = a[min];
            a[min] = t;
        }
        for (int j = 0; j < N - 1; j++)
            assert a[j] <= a[j + 1];
    }

    public static void selectSort_ax(final int N, int[] a) {
        for (int j = 0; j < N - 1; j++) {
            int min = j;
            for (int i = j + 1; i < N; i++) {
                if (a[min] > a[i])
                    min = i;
            }
            int t = a[j];
            a[j] = a[min];
            a[min] = t;
        }
        assert Axioms.isSorted(a, 0, N);
    }
    
    public static void selectSort_bug1(final int N, int[] a) {
        for (int j = 0; j < N - 1; j++) {
            int min = j;
            for (int i = j + 1; i < N; i++) {
                if (a[min] > a[i])
                    min = i;
            }
            int t = a[j];
            a[j] = a[min];
            a[min] = t;
        }
        for (int j = 0; j < N - 1; j++)
            assert a[j] < a[j + 1]; // bug
    }

    public static void selectSort_bug2(final int N, int[] a) {
        for (int j = 0; j < N - 2; j++) { // bug
            int min = j;
            for (int i = j + 1; i < N; i++) {
                if (a[min] > a[i])
                    min = i;
            }
            int t = a[j];
            a[j] = a[min];
            a[min] = t;
        }
        for (int j = 0; j < N - 1; j++)
            assert a[j] <= a[j + 1];
    }

    public static void selectSort_bug3(final int N, int[] a) {
        for (int j = 0; j < N - 1; j++) {
            int min = j;
            for (int i = j + 2; i < N; i++) { // bug
                if (a[min] > a[i])
                    min = i;
            }
            int t = a[j];
            a[j] = a[min];
            a[min] = t;
        }
        for (int j = 0; j < N - 1; j++)
            assert a[j] <= a[j + 1];
    }

    public static int sqRoot(final int N) {
        int r = 1;
        int q = N;
        while (r + 1 < q) {
            int p = (r + q) / 2;
            if (N < p * p) {
                q = p;
            } else {
                r = p;
            }
        }
        assert r * r <= N && (r + 1) * (r + 1) > N;
        return r;
    }
    
    public static int sqRoot_bug1(final int N) {
        int r = 0; // BUG
        int q = N;
        while (r + 1 < q) {
            int p = (r + q) / 2;
            if (N < p * p) {
                q = p;
            } else {
                r = p;
            }
        }
        assert r * r <= N && (r + 1) * (r + 1) > N;
        return r;
    }
    
    public static int sqRoot_bug2(final int N) {
        int r = 1;
        int q = N;
        while (r + 1 < q) {
            int p = (r + q) / 2;
            if (N < p * p) {
                r = p; // BUG
            } else {
                q = p; // BUG
            }
        }
        assert r * r <= N && (r + 1) * (r + 1) > N;
        return r;
    }

    public static int gcd(final int a, final int b) {
        int aa = a, bb = b;
        int r;
        do {
            r = aa % bb;
            aa = bb;
            bb = r;
        } while (r > 0);
        assert a % aa == 0 && b % aa == 0;
        for (int i = aa + 1; i <= a/2 && i <= b/2; i++)
            assert a % i != 0 || b % i != 0;
        return aa;
    }
    
    public static int gcd_bug1(final int a, final int b) {
        int aa = a, bb = b;
        int r;
        do {
            r = aa % bb;
            aa = bb;
            // bb = r; // BUG
        } while (r > 0);
        assert a % aa == 0 && b % aa == 0;
        for (int i = aa + 1; i <= a/2 && i <= b/2; i++)
            assert a % i != 0 || b % i != 0;
        return aa;
    }

    public static void test1() {
        int i = 0;
        if (i < 1) {
            i = i + 4;
            if (i > 5)
                i = 10;
            else
                i = 20;
        }
        assert i == 20;
    }

}
