package softmc.logic.visitors;

import korat.utils.reflection.ReflectionUtils;
import korat.utils.visitors.IVisitor;
import korat.utils.visitors.VisitorUtils;
import softmc.logic.App;
import softmc.logic.BinTerm;
import softmc.logic.Factory;
import softmc.logic.N_aryTerm;
import softmc.logic.Term;
import softmc.logic.UnaryTerm;
import softmc.logic.VarModTerm;

public class CloneVisitor implements IVisitor<Term> {

    public Term clone(Term termToClone) {
        return VisitorUtils.visit(termToClone, this);
    }

    public Term visitUnaryTerm(UnaryTerm term) {
        Term innerTerm = VisitorUtils.visit(term.getTerm(), this);
        return ReflectionUtils.newInstance(term.getClass(), Term.class, innerTerm);
    }

    public Term visitBinTerm(BinTerm term) {
        Term lhs = VisitorUtils.visit(term.getLhs(), this);
        Term rhs = VisitorUtils.visit(term.getRhs(), this);
        return ReflectionUtils.newInstance(term.getClass(), Term.class, Term.class, lhs, rhs);
    }
    
    public Term visitApp(App term) {
        Term[] terms = term.getTerms();
        Term[] newTerms = new Term[terms.length];
        for (int i = 0; i < terms.length; i++) {
            newTerms[i] = VisitorUtils.visit(terms[i], this);
        }
        return Factory.instance.mkApp(term.getAppName(), term.getRetSort(), newTerms);
    }

    public Term visitN_aryTerm(N_aryTerm term) {
        Term[] terms = term.getTerms();
        Term[] newTerms = new Term[terms.length];
        for (int i = 0; i < terms.length; i++) {
            newTerms[i] = VisitorUtils.visit(terms[i], this);
        }
        return ReflectionUtils.newInstance(term.getClass(), Term[].class, newTerms);
    }

    public Term visitVarModTerm(VarModTerm term) {
        return term;
    }

    public Term visitTerm(Term term) {
        return term;
    }

}
