package softmc.logic.visitors;

import softmc.logic.BinTerm;
import softmc.logic.N_aryTerm;
import softmc.logic.Term;
import softmc.logic.UnaryTerm;
import softmc.logic.VarModTerm;
import korat.utils.visitors.IVisitor;
import korat.utils.visitors.VisitorUtils;

public class DFSVisitor<T> implements IVisitor<T> {

    private final IVisitor<T> otherVisitor; 
    
    public DFSVisitor(IVisitor<T> otherVisitor) {
        this.otherVisitor = otherVisitor;
    }

    public void visitUnaryTerm(UnaryTerm term) {
        VisitorUtils.visit(term, otherVisitor);
        VisitorUtils.visit(term.getTerm(), this);
    }
    
    public void visitBinTerm(BinTerm term) {
        VisitorUtils.visit(term, otherVisitor);
        VisitorUtils.visit(term.getLhs(), this);
        VisitorUtils.visit(term.getRhs(), this);
    }
 
    public void visitN_aryTerm(N_aryTerm term) {
        VisitorUtils.visit(term, otherVisitor);
        for (Term t : term.getTerms()) {
            VisitorUtils.visit(t, this);
        }
    }
    
    public void visitVarModTerm(VarModTerm term) {
        VisitorUtils.visit(term, otherVisitor);
        VisitorUtils.visit(term.getVar(), this);
    }
    
    public void visitTerm(Term term) {
        VisitorUtils.visit(term, otherVisitor);
    }
}
