package softmc.logic.visitors;

import java.io.PrintStream;

import korat.utils.visitors.IVisitor;
import korat.utils.visitors.VisitorUtils;
import softmc.logic.BinTerm;
import softmc.logic.N_aryTerm;
import softmc.logic.Term;
import softmc.logic.UnaryTerm;

public class PrettyPrinter implements IVisitor<Void> {

    private StringBuilder sb; 
    private String indent = "";
    private String tab = "  ";
    
    public void print(Term t, PrintStream ps) {
        sb = new StringBuilder();
        VisitorUtils.visit(t, this);
        ps.print(sb.toString());
    }
    
    public void visitUnaryTerm(UnaryTerm term) {
        sb.append(String.format("%s(%s\n", indent, term.getOp()));
        indent += tab;
        VisitorUtils.visit(term.getTerm(), this);
        sb.append(")\n");
        indent = indent.substring(tab.length());
    }

    public void visitBinTerm(BinTerm term) {
        sb.append(String.format("%s(%s\n", indent, term.getOp()));
        indent += tab;
        VisitorUtils.visit(term.getLhs(), this);
        sb.append("\n");
        VisitorUtils.visit(term.getRhs(), this);
        sb.append(")");
        indent = indent.substring(tab.length());
    }

    public void visitN_aryTerm(N_aryTerm term) {
        sb.append(String.format("%s(%s\n", indent, term.getOp()));
        indent += tab;
        int len = term.getTerms().length;
        for (int i = 0; i < len; i++) {
            VisitorUtils.visit(term.getTerm(i), this);
            if (i < len - 1)
                sb.append("\n");
            else 
                sb.append(")");
        }
        indent = indent.substring(tab.length());
    }

    public void visitTerm(Term term) {
        sb.append(indent).append(term.toString());
    }
}
