package softmc.logic.visitors;

import korat.utils.visitors.IVisitor;
import korat.utils.visitors.VisitorUtils;
import softmc.logic.BinTerm;
import softmc.logic.N_aryTerm;
import softmc.logic.Sort;
import softmc.logic.Term;
import softmc.logic.UnaryTerm;
import softmc.logic.Var;
import softmc.logic.VarModTerm;

public class VarExtractor implements IVisitor<Var> {

    private Sort typeFilter; 
    
    public Var extractVar(Term t, Sort typeFilter) {
        this.typeFilter = typeFilter;
        return VisitorUtils.visit(t, this);
    }
    
    public Var visitUnaryTerm(UnaryTerm term) {
        return VisitorUtils.visit(term.getTerm(), this);
    }

    public Var visitBinTerm(BinTerm term) {
        Var v1 = VisitorUtils.visit(term.getLhs(), this);
        Var v2 = VisitorUtils.visit(term.getRhs(), this);
        if (v1 == null)
            return v2;
        else if (v2 != null)
            throw new RuntimeException("Found multiple variables of type: " + typeFilter);
        else 
            return v1;
    }

    public Var visitN_aryTerm(N_aryTerm term) {
        Var ret = null;
        for (Term t : term.getTerms()) {
            Var v = VisitorUtils.visit(t, this);
            if (v != null) {
                if (ret == null)
                    ret = v; 
                else if (!ret.getVarName().equals(v.getVarName())) { 
                    throw new RuntimeException("Found multiple variables: " + term);
                }
            }
        }
        return ret;
    }

    public Var visitVarModTerm(VarModTerm term) {
        Var v = term.getVar();
        return visitVar(v);
    }
    
    public Var visitVar(Var var) {
        if (typeFilter == null || var.getVarType().getSortName().equals(typeFilter.getSortName()))
            return var; 
        else 
            return null;
    }

    public Term visitTerm(Term term) {
        return null;
    }
    
}
