package softmc.logic.visitors;

import korat.utils.func.Pred;
import korat.utils.reflection.ReflectionUtils;
import korat.utils.visitors.IVisitor;
import korat.utils.visitors.VisitorUtils;
import softmc.logic.BinTerm;
import softmc.logic.N_aryTerm;
import softmc.logic.Term;
import softmc.logic.UnaryTerm;
import softmc.logic.Var;
import softmc.logic.VarModTerm;

/**
 * This visitor substitutes var modifiers (not free variables) with the given another
 * var modifier term.
 */
@Deprecated
class VarRewrapper implements IVisitor<Term> {

    public static final Pred<Var> allVarsPred = new Pred<Var>() {
        public Boolean exe(Var var) {
            return true;
        }
    };

    private Class<? extends VarModTerm> substituteWith;
    private Pred<Var> varPred;
    private Class<? extends VarModTerm> varModFilter;

    public Term substituteAllVars(Term termToVisit, Class<? extends VarModTerm> substituteWith) {
        return substitute(termToVisit, allVarsPred, VarModTerm.class, substituteWith);
    }

    /**
     * @param termToVisit
     * @param varPred
     * @param varModFilter
     * @param substituteWith
     *            - null means just remove the var modifier
     * @param clone
     * @return
     */
    public Term substitute(Term termToVisit, Pred<Var> varPred,
            Class<? extends VarModTerm> varModFilter, Class<? extends VarModTerm> substituteWith) {
        this.varPred = varPred;
        this.varModFilter = varModFilter;
        this.substituteWith = substituteWith;
        return VisitorUtils.visit(termToVisit, this);
    }

    public Term visitUnaryTerm(UnaryTerm term) {
        Term innerTerm = VisitorUtils.visit(term.getTerm(), this);
        term.setTerm(innerTerm);
        return term;
    }

    public Term visitBinTerm(BinTerm term) {
        Term lhs = VisitorUtils.visit(term.getLhs(), this);
        Term rhs = VisitorUtils.visit(term.getRhs(), this);
        term.setLhs(lhs);
        term.setRhs(rhs);
        return term;
    }

    public Term visitN_aryTerm(N_aryTerm term) {
        Term[] terms = term.getTerms();
        Term[] newTerms = new Term[terms.length];
        for (int i = 0; i < terms.length; i++) {
            newTerms[i] = VisitorUtils.visit(terms[i], this);
        }
        term.setTerms(newTerms);
        return term;
    }

    public Term visitVarModTerm(VarModTerm term) {
        Var var = term.getVar();
        if (!varPred.exe(var))
            return term;
        if (!varModFilter.isAssignableFrom(term.getClass()))
            return term;
        if (substituteWith == null)
            return var;
        else
            return ReflectionUtils.newInstance(substituteWith, Var.class, var);
    }

    public Term visitTerm(Term term) {
        return term;
    }

}
