package softmc.state;

import softmc.logic.Factory;
import softmc.logic.Term;

public class StateTransition {

    private BlockState source, dest;
    private Term guard = Factory.instance.mkBool(true);
    private SymState effects = new SymState(); 
    
    public StateTransition(BlockState source, BlockState dest, Term guard) {
        this.source = source;
        this.dest = dest;
        this.guard = guard;
    }
    
    public StateTransition(BlockState source, BlockState dest) {
        this(source, dest, Factory.instance.mkBool(true));
    }

    public Term getGuard() {
        return guard;
    }

    public void setGuard(Term guard) {
        this.guard = guard;
    }

    public BlockState getSource() {
        return source;
    }

    public BlockState getDest() {
        return dest;
    }

    public void setSource(BlockState source) {
        this.source = source;
    }

    public void setDest(BlockState dest) {
        this.dest = dest;
    }

    public SymState getEffects() {
        return effects;
    }

    /**
     * Appends the given symbolic state to this transition's effects map as if this 
     * transition's effects were applied first and then the symbolic state was computed. 
     * In effect, it replaces all "old var" term in the given symbolic state with the 
     * value which that variable has in the effects map of this transition.  
     * 
     * It doesn't modify the given symState object. 
     */
    public void appendEffects(SymState symState) {
        effects = SymState.mergeStates(effects, symState);
    }

    @Override
    public String toString() {
        return String.format("[%s] %s", guard, effects.toString().replaceAll("\\n", ";"));
    }

}