package softmc.examples;

import static org.junit.Assert.assertArrayEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNull;

import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

import org.junit.Test;

import softmc.GlobalOptions;
import softmc.ModelChecker;
import softmc.state.GlobalState;
import softmc.tools.Interpreter;
import softmc.tools.Interpreter.ExePath;

public abstract class TestSort {

    @Test public void test0() throws Exception { testAndCheckOk(new int[] { 2, 2, 2 }); }
    @Test public void test1() throws Exception { testAndCheckOk(new int[] { 2, 3, 4 }); }
    @Test public void test2() throws Exception { testAndCheckOk(new int[] { 2, 2, 4 }); }
    @Test public void test3() throws Exception { testAndCheckOk(new int[] { 2, 3, 2 }); }
    @Test public void test4() throws Exception { testAndCheckOk(new int[] { 2, 3, 3 }); }
    @Test public void test5() throws Exception { testAndCheckOk(new int[] { 6, 7, 4 }); }
    @Test public void test6() throws Exception { testAndCheckOk(new int[] { 6, 5, 4 }); }
    @Test public void test7() throws Exception { testAndCheckOk(new int[] { 6, 6, 4 }); }
    @Test public void test8() throws Exception { testAndCheckOk(new int[] { 6, 5, 6 }); }
    @Test public void test9() throws Exception { testAndCheckOk(new int[] { 6, 4, 4 }); }
    
    
    protected void testAndCheckOk(int[] a) throws Exception {
        GlobalOptions.__elim_strategy = GlobalOptions.DFS_STRATEGY;
        assertOkAndSorted(runSort(a), a);
        
        GlobalOptions.__elim_strategy = GlobalOptions.REV_DFS_STRATEGY;
        assertOkAndSorted(runSort(a), a);
    }
    
    protected void assertOkAndSorted(ExePath path, int[] a) {
        assertNull(path.getErrorMessage());
        assertFalse(path.isInErrorState());
        int[] res = (int[]) path.getLast().getValue().get("a1");
        int[] sortedArr = Arrays.copyOf(a, a.length);
        Arrays.sort(sortedArr);
        assertArrayEquals(sortedArr, res);        
    }

    protected ExePath runSort(int[] a) throws Exception {
        String className = Examples.class.getName();
        String methodName = getSortMethodName();
        ModelChecker mc = new ModelChecker();
        GlobalState gs = mc.convertToState(className, methodName);
        Map<String, Object> vals = new HashMap<String, Object>();
        vals.put("N0", a.length);
        vals.put("a1", a);
        return new Interpreter(gs, vals).go();
    }
    

    protected abstract String getSortMethodName();
    
}
